/***************************************************************************/
/*                                                                         */
/*  File: main.cpp                                                         */
/*  Date: 22-12-06                                                         */
/*                                                                         */
/***************************************************************************/

#define STRICT					// Strict compiling
#define WIN32_LEAN_AND_MEAN		// Tells compiler to produce tigher/smaller code 


#include <windows.h>            
#include <assert.h>
#include <math.h>				// abs(..)


//#include <GL/gl.h>
#include <GL/glu.h>             // OpenGL Header File
//#include <GL/glaux.h>

#pragma comment(lib, "opengl32.lib") // OpenGL librarys
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glaux.lib")



/*****************************************************************************/
/*
   GLOBALS
   As much as we all hate them, its sometimes a little easier...so we've got
   a few globals for the essential of our code.
/*
/*****************************************************************************/

HDC       g_hdc;                  // global device context
HGLRC     g_hrc;                  // global rendering context

HWND      g_hwnd = NULL;          // handle of our window


// The next 5 variables are used to keep track of mouse input - so we can
// rotate our object with the left moust button down, and zoom using the 
// right mouse button.  And of course a velocity variable which allows
// us to drag and release our shape so it builds up speed and continues to
// rotate
float	  g_fSpinX			= 0.0f; // Left mouse button rotation
float	  g_fSpinY			= 0.0f;

float     g_fVelX           = 0.2f; // Drag and rotate 
float     g_fVelY           = 0.05f;

float	  g_fZoom			= -20.0f; // Right mouse button zoom value

// Defined at the top of render.cpp
extern	  bool g_bOrbitCam;

/***************************************************************************/
// PROTOTYPES                                                              //
/***************************************************************************/
void startUp(void);      // - called at the start of our program
void render(void);       // - our mainloop..called over and over again
void create(void);		 // - called once at startup, for any game data
void release(void);		 // - called once at destroy, for any game data
void shutDown(void);     // - last function we call before we end!


/***************************************************************************/
/*                                                                         */
/* Handle all messages for the main window here                            */
/*                                                                         */
/***************************************************************************/
long _stdcall MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	// We declare the variables as static so we don't have to declare them
	// global!
    static POINT ptLastMousePosit;      // Last mouse position
    static POINT ptCurrentMousePosit;   // Current mouse position
    static bool bMousing;				// Left mouse butotn is pressed
	static bool bZooming;				// Right mouse button is pressed

    switch( uMsg )
    {
		case WM_CHAR:					// received a character message
		{
			// We do a message check, as the probing of the keyboard in main
			// render loop is to fast, and leads to flickering!

			switch(wParam)				// check what was pressed through wParam
			{
				// Toggle the camera view
				case 'C':
				case 'c':
				{
					g_bOrbitCam = !g_bOrbitCam;
				}
				break;
			}
		}
		break;
	
        case WM_KEYDOWN:
        {
            switch( wParam )
            {
                case VK_ESCAPE:         // Exit on pressing Escape Key
                    PostQuitMessage(0);
                    break;
			}
        }
        break;

        case WM_LBUTTONDOWN:
        {
            ptLastMousePosit.x = ptCurrentMousePosit.x = LOWORD (lParam);
            ptLastMousePosit.y = ptCurrentMousePosit.y = HIWORD (lParam);
            bMousing = true;
        }
        break;

        case WM_LBUTTONUP:
        {
            bMousing = false;
        }
        break;

		case WM_RBUTTONDOWN:
        {
            ptLastMousePosit.x = ptCurrentMousePosit.x = LOWORD (lParam);
            ptLastMousePosit.y = ptCurrentMousePosit.y = HIWORD (lParam);
            bZooming = true;
        }
        break;

		case WM_RBUTTONUP:
        {
			bZooming = false;
        }

        case WM_MOUSEMOVE:
        {
            ptCurrentMousePosit.x = LOWORD (lParam);
            ptCurrentMousePosit.y = HIWORD (lParam);

            if( bMousing )
            {
                g_fSpinX -= (ptCurrentMousePosit.x - ptLastMousePosit.x);
                g_fSpinY -= (ptCurrentMousePosit.y - ptLastMousePosit.y);
				
				float dx = (float)(ptCurrentMousePosit.x - ptLastMousePosit.x);
				float dy = (float)(ptCurrentMousePosit.y - ptLastMousePosit.y);
				
				g_fVelX = (float)dx/2.0f;
				g_fVelY = (float)dy/2.0f;

				// We detect that only when a user drags and releases
				// the mouse button above a certain speed will the shape
				// actually rotate by itself.
				if( abs(g_fVelX) < 0.6f ) g_fVelX = 0.0f;
				if( abs(g_fVelY) < 0.6f ) g_fVelY = 0.0f;
            }

			if( bZooming )
            {
				// 0.2 is so the zoom isn't so fast!  Makes it more user friendly
                g_fZoom += (ptCurrentMousePosit.y - ptLastMousePosit.y) * 0.2f;
            }

            
            ptLastMousePosit.x = ptCurrentMousePosit.x;
            ptLastMousePosit.y = ptCurrentMousePosit.y;
        }
	}//End switch(..)

    if(uMsg == WM_DESTROY)
    {
            // We destroy any directx memory we allocated etc, 
            // Tidy up before leaving.
		    shutDown();

            //UnregisterClass( "DirectX3D", winClass.hInstance );
            PostQuitMessage(0);
			return 0;
    }

    return (long)DefWindowProc(hWnd, uMsg, wParam, lParam);

}//End MsgProc(..)


/***************************************************************************/
/*                                                                         */
/* Program entry point.                                                    */
/*                                                                         */
/***************************************************************************/
int _stdcall WinMain(HINSTANCE i, HINSTANCE, char* k, int) 
{
    MSG msg;
    char szname[] = "Textured Planet Earth";
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      szname, NULL };
    RegisterClassEx( &wc );
    g_hwnd = CreateWindowEx(WS_EX_APPWINDOW,
                              szname, 
							  szname, 
                              WS_OVERLAPPEDWINDOW,//for fullscreen make into WS_POPUP
                              50, 50, 640+50,480+50,    //for full screen GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
                              GetDesktopWindow(), NULL, wc.hInstance, NULL);
    
    // Initilise our graphics code here!
	startUp();

    ShowWindow(g_hwnd, SW_SHOW);
    UpdateWindow(g_hwnd);     
 
    // Message loop. Note that this has been modified to allow
    // us to execute if no messages are being processed.
    while(1)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
            if (!GetMessage(&msg, NULL, 0, 0))
                break;

			TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
		else
		{
			static float oldTime = (float)GetTickCount();
			float nowTime = (float)GetTickCount() + 1.0f;
			float delay = nowTime - oldTime;
			if ( delay > 20)
			{
				oldTime = nowTime;

				render();
				SwapBuffers(g_hdc);
			}
		}
    }
    return 0;
}// End WinMain(..)



/***************************************************************************/
/*                                                                         */
/* startUp()                                                               */
/* Called once at the start of our program to setup graphics.              */
/*                                                                         */
/***************************************************************************/
void startUp( void )
{

	PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		24,
		0,0,0,0,0,0,0,0,0,0,0,0,0, // useles parameters
		16,
		0,0,0,0,0,0,0
	};


	GLuint  pixelFormat;

	g_hdc = GetDC(g_hwnd);

	pixelFormat = ChoosePixelFormat(g_hdc, &pfd);
	SetPixelFormat(g_hdc, pixelFormat, &pfd);

	g_hrc = wglCreateContext(g_hdc);

	wglMakeCurrent(g_hdc, g_hrc);

	ShowWindow(g_hwnd, SW_SHOW);
	SetForegroundWindow(g_hwnd);
	SetFocus(g_hwnd);

	glEnable(GL_DEPTH_TEST);

	create();
}//End init(..)




/***************************************************************************/
/*                                                                         */
/* shutDown()                                                              */
/* Release any allocated memory, andy directx api's, resources etc before  */
/* our program finally terminates.                                         */
/*                                                                         */
/***************************************************************************/
void shutDown( void )
{
	release();

    wglMakeCurrent(NULL,NULL);

	wglDeleteContext(g_hrc);

	ReleaseDC(g_hwnd, g_hdc);

	DestroyWindow(g_hwnd);

}// End shutDown(..)



/***************************************************************************/
/*                                                                         */
/* DebugPrint()                                                            */
/*  Used to display simple debug information to the screen.  Not for       */
/*  optimised use - due to the fact that it creates/releases resources     */
/*  each time round it is called.                                          */
/*                                                                         */
/***************************************************************************/
void DebugPrint( char * str, float x, float y)
{
	/*
	GLuint fontbase; 

	HFONT   font;                              // Windows Font ID 
    HFONT   oldfont;                           // Used For Good House Keeping 

	fontbase = glGenLists(256);					// Storage For 96 Characters 

	font = CreateFont(-13,						// Height Of Font 
						0,						// Width Of Font 
						0,						// Angle Of Escapement 
						0,						// Orientation Angle 
						FW_BOLD,				// Font Weight 
						FALSE,					// Italic 
						FALSE,					// Underline 
						FALSE,					// Strikeout 
						ANSI_CHARSET,			// Character Set Identifier 
						OUT_TT_PRECIS,			// Output Precision 
						CLIP_DEFAULT_PRECIS,		// Clipping Precision 
						ANTIALIASED_QUALITY,		// Output Quality 
						FF_DONTCARE|DEFAULT_PITCH,	// Family And Pitch 
						"Courier New");	// Font Name 

	oldfont = (HFONT)SelectObject(g_hdc, font);		// Selects The Font We Want 
	wglUseFontBitmaps(g_hdc, 0, 256, fontbase);		// Builds 256 Characters 
	SelectObject(g_hdc, oldfont);					// Selects The Font We Want 
	DeleteObject(font);								// Delete The Font 


	glMatrixMode(GL_MODELVIEW);		
	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	//glLoadIdentity();	
	glOrtho(0, 640, 0, 480, -1, 1);

	glScalef(1.0f, 1.0f, 1.0f);
	glLoadIdentity();					
	glTranslatef(0.0f,450.0f,-1.0f);
	glColor3f (1.0f, 1.0f, 1.0f);


	glRasterPos2f (x,   // x pos
				   -y); // y pos, negatated so it goes from top to bottom


	glPushAttrib(GL_LIST_BIT);						// Pushes The Display List Bits 
	glListBase(fontbase);							// Sets The Base Character to 0 
	glCallLists((GLsizei)strlen(str), GL_UNSIGNED_BYTE, str);   // Draws The Display List Text 
	glPopAttrib();									// Pops The Display List Bits 


	glDeleteLists(fontbase, 256);					// Delete All 256 Characters 
	*/
}//End DebugPrint(..)


/*
Note:
Alternative Text Out method using glut
	//glLoadIdentity ();
	//glColor3f (1.0F, 1.0F, 1.0F);
	//glRasterPos2f (-0.80F, -0.80F);

	//char str[] = "test";
	//  unsigned int i;
	//  for (i = 0; i < strlen (str); i++)
	//	glutBitmapCharacter (GLUT_BITMAP_HELVETICA_18, str[i]);
*/

