/***************************************************************************/
/*                                                                         */
/*  File: render.cpp                                                       */
/*  Date: 22-12-06                                                         */
/*                                                                         */
/***************************************************************************/
/*

*/
/***************************************************************************/


#include <math.h>		// cosf(..) sinf(..), sqrt(..)
#include <stdio.h>      // sprintf(..)

#include <windows.h>    // opengl has issues if we don't include it!
//#include <GL/gl.h>
#include <GL/glu.h>		// gluLookAt(..), glPerspective(..)
//#include <GL/glut.h>  

 #include <gl/glaux.h>	// auxDIBImageLoad(..)



#include "vector3.h"	// VECTOR3

// OpenGL librarys
#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "glaux.lib")


// Declared in main.cpp - used to display simple debug information to the screen
// such as variable values etc
extern void DebugPrint( char * str, float x, float y);

extern float			g_fSpinX;	// main.cpp - used for x mouse rotation
extern float			g_fSpinY;   // main.cpp - used for y mouse rotation

extern float			g_fVelX;    // main.cpp - dynamic rotation effect from mouse
extern float			g_fVelY;    // main.cpp - dynamic rotation effect from mouse

extern float			g_fZoom;    // main.cpp - right mouse button zooming

extern HWND				g_hwnd;		// main.cpp - handle of our window


static bool			g_bWire			 = false;	// Wire frame rendering on/off
static bool			g_bWire_On_Solid = true;	// Wire frame on top of a solid render
static bool			g_bSurface		 = true;	// Turn surface rendering on/off

	   bool			g_bOrbitCam		 = false;   // Use camera that orbits (toggle in main msg loop)
static VECTOR3		g_OribitPos;
static VECTOR3		g_OribitPosNext;
static float		g_OrbitSpeed	 = 0.1f;	// Speed we go around the planet

static int			g_NumLatitudes  = 20;		// Number of segments that make up ring
static int			g_NumLongitudes = 20;		// Segments for our shape length   

static GLuint		g_textureID		= 0;		// Texture ID Number used for OpenGL
static bool			g_have_texture	= true;		// If we managed to load a tex in

// Various math define values for our sphere
static double		g_PI			= 3.14159265359;
static double		g_TWOPI			= 6.28318530718;
static double		g_DE2RA			= 0.01745329252;
static double		g_RA2DE			= 57.2957795129;
static double		g_FLATTENING	= 1.0/298.26;
static double		g_PIOVER2		= 1.570796326795;


enum ORBIT
{
	ORBITA = 0,
	ORBITB,
	ORBITC,
};
static ORBIT		g_OrbitSelected = ORBITA;


/***************************************************************************/
/*                                                                         */
/*  Name: KeyBoard()                                                       */
/*  Date: 22-12-06                                                         */
/*                                                                         */
/*  Used to accept keyboard input and alter some global variables - used   */
/*  for simple analysis of our code - turning wire frame rendering on/off  */
/*  adjusting variables which are used in the generation of our shape etc. */
/*                                                                         */
/***************************************************************************/
void KeyBoard()
{
	// Controlling the rendering
	if( GetKeyState('W')		& 0x80 ) g_bWire=!g_bWire;
	if( GetKeyState('Q')		& 0x80 ) g_bWire_On_Solid=!g_bWire_On_Solid;
	if( GetKeyState('O')		& 0x80 ) g_bSurface=!g_bSurface;
//	if( GetKeyState('C')		& 0x80 ) g_bOrbitCam=!g_bOrbitCam;

	if( GetKeyState('Z')		& 0x80 )
	{
		g_OrbitSpeed=min(10.0f, g_OrbitSpeed+0.01f);
	}
	if( GetKeyState('X')		& 0x80 ) g_OrbitSpeed=max(0.1f, g_OrbitSpeed-0.01f);

	// Controlling the mesh detail
	if( GetKeyState(VK_RIGHT)	& 0x80 )  g_NumLatitudes=min(120,g_NumLatitudes +1);
	if( GetKeyState(VK_LEFT)	& 0x80 )  g_NumLatitudes=max(4,  g_NumLatitudes -1);

	if( GetKeyState(VK_UP)		& 0x80 )  g_NumLongitudes=min(180,g_NumLongitudes +1);
	if( GetKeyState(VK_DOWN)	& 0x80 )  g_NumLongitudes=max(15, g_NumLongitudes -1);

	if( GetKeyState(VK_F1)      & 0x80 )  g_OrbitSelected = ORBITA;
	if( GetKeyState(VK_F3)      & 0x80 )  g_OrbitSelected = ORBITB;
	if( GetKeyState(VK_F2)      & 0x80 )  g_OrbitSelected = ORBITC;

}//End KeyBoard()




// Simple stucture which we use to define each Vertex
struct stCUSTOMVERTEX
{
    float x, y, z;			//Vertex Position 
    float nx, ny, nz;       //Vertex Normal
    float r,g,b;			//Colour
};

// We use basic matrices for rotation around an arbitrary axis,
// and we define our matrix here
struct MATRIX4
{
	float	_11, _12, _13, _14,
			_21, _22, _23, _24,
			_31, _32, _33, _34,
			_41, _42, _43, _44;

};//End MATRIX4


/***************************************************************************/
/*                                                                         */
/*  Name: create(..)                                                       */
/*  Date: 22-12-06                                                         */
/*                                                                         */
/***************************************************************************/
void create()
{
	AUX_RGBImageRec* image = auxDIBImageLoad("land_ocean.bmp");

	if (image==NULL)
	{
		g_have_texture = false;
		return;
	}


	// Create One Textures
	glGenTextures(	1,							// # Of Texture Names/ID To Create
					&g_textureID);				// Generated Texture ID

	glBindTexture(	GL_TEXTURE_2D,				// Bind A 2D Texture 
					g_textureID);				// Bind Using This Texture ID

	glTexParameteri(GL_TEXTURE_2D,				// Set 2D Texture Parameter
					GL_TEXTURE_MIN_FILTER,		// Select Minification Filter Type
//					GL_NEAREST);				// GL_NEAREST (Blocky & Fastest)
					GL_LINEAR);	

	glTexParameteri(GL_TEXTURE_2D,				// Set 2D Texture Parameter
						GL_TEXTURE_MAG_FILTER,	// Select Magnification Filter Type
						GL_LINEAR);				// GL_LINEAR (Smoothest & Slowest)

	glTexImage2D(	GL_TEXTURE_2D,				// GLenum target,
					0,							// GLint level,
					GL_RGB8,					// GLint internalformat,
					image->sizeX,				// GLsizei width,
					image->sizeY,				// GLsizei height,
					0,							// GLint border,
					GL_RGB,						// GLenum format,
					GL_UNSIGNED_BYTE,(GLvoid *)	// GLenum type,
					image->data);				// const GLvoid *pixels

	if (image)									// If Texture Exists
	{								
		if (image->data)
		{
			delete image->data;					// If Texture Image Exists
		}
		delete image;
		image=NULL;
	}
}// End create(..)

/***************************************************************************/
/*                                                                         */
/*  Name: release(..)                                                      */
/*  Date: 22-12-06                                                         */
/*                                                                         */
/***************************************************************************/
void release()
{
	// Release any resource that need to be released once before the program
	// exit - non yet!...but we have the function incase :)
}// End release(..)


/***************************************************************************/
/*                                                                         */
/*  Name: DrawTexSphere(..)                                                */
/*  Date: 22-12-06                                                         */
/*                                                                         */
/*  About: Main function called from the render loop, which generates and  */
/*  draws the textured sphere.                                             */
/*                                                                         */
/***************************************************************************/
void DrawTexSphere()
{
	if (g_have_texture)
	{
		glBindTexture(	GL_TEXTURE_2D,g_textureID);
		glEnable( GL_TEXTURE_2D );
	}
	else
	{
	  glDisable( GL_TEXTURE_2D );
	}

	glBegin( GL_TRIANGLES );

	float start_lat = -90;
	float start_lon = 0.0;
	float R = 1.0;

	double lat_incr = 180.0 / g_NumLatitudes;
	double lon_incr = 360.0 / g_NumLongitudes;

	int row, col;

	double phi1, phi2;
	double u[3];
	double v[3];
	double w[3];

	for (col = 0; col < g_NumLongitudes; col++)
	{
		phi1 = (start_lon + col * lon_incr) * g_DE2RA;
		phi2 = (start_lon + (col + 1) * lon_incr) * g_DE2RA;

		for (row = 0; row < g_NumLatitudes; row++)
		{
			double theta1 = (start_lat + row * lat_incr) * g_DE2RA;
			double theta2 = (start_lat + (row + 1) * lat_incr) * g_DE2RA;

			u[0] = R * cos(phi1) * cos(theta1);    //x
			u[1] = R * sin(theta1);                //y
			u[2] = R * sin(phi1) * cos(theta1);    //z

			v[0] = R * cos(phi1) * cos(theta2);    //x
			v[1] = R * sin(theta2);                //y
			v[2] = R * sin(phi1) * cos(theta2);    //z

			w[0] = R * cos(phi2) * cos(theta2);    //x
			w[1] = R * sin(theta2);                //y
			w[2] = R * sin(phi2) * cos(theta2);    //z

			// Normalize Vector
			double n[3];
			for (int i=0; i<3; i++)
			{
				n[i] = sqrt( u[i]*u[i] + v[i]*v[i] + w[i]*w[i] );
			}


			glNormal3dv(n);
			glTexCoord2d((180.0 - phi1*g_RA2DE)/360.0,
						 (theta1 + g_PIOVER2)*g_RA2DE/180.0);
			glVertex3dv(u);
			glTexCoord2d((180.0 - phi1*g_RA2DE)/360.0,
						 (theta2 + g_PIOVER2)*g_RA2DE/180.0);
			glVertex3dv(v);
			glTexCoord2d((180.0 - phi2*g_RA2DE)/360.0,
						 (theta2 + g_PIOVER2)*g_RA2DE/180.0);
			glVertex3dv(w);

			v[0] = R * cos(phi2) * cos(theta1);    //x
			v[1] = R * sin(theta1);                //y
			v[2] = R * sin(phi2) * cos(theta1);    //z

			for (int i=0; i<3; i++)
			{
				n[i] = sqrt( u[i]*u[i] + v[i]*v[i] + w[i]*w[i] );
			}

			glNormal3dv(n);
			glTexCoord2d((180.0 - phi1*g_RA2DE)/360.0,
						 (theta1 + g_PIOVER2)*g_RA2DE/180.0);
			glVertex3dv(u);
			glTexCoord2d((180.0 - phi2*g_RA2DE)/360.0,
						 (theta2 + g_PIOVER2)*g_RA2DE/180.0);
			glVertex3dv(w);
			glTexCoord2d((180.0 - phi2*g_RA2DE)/360.0,
						 (theta1 + g_PIOVER2)*g_RA2DE/180.0);
			glVertex3dv(v);

			glDisable( GL_TEXTURE_2D );
		}
	}

	glEnd();

}// End DrawTexSphere(..)


/***************************************************************************/
/*                                                                         */
/*  Name: UpdateCamera(..)                                                 */
/*  Date: 22-12-06                                                         */
/*                                                                         */
/***************************************************************************/
void UpdateCamera()
{	// Setting World/Camera Constants	
	RECT rect;
	::GetClientRect(g_hwnd, &rect);

	int w = rect.right - rect.left;
	int h = rect.bottom - rect.top;

	if (w==0 || h==0)
	{
		w=640;
		h=480;
	}

	// Current viewport:x, y, width, height
	glViewport(0, 0, w, h);    
 
	// Next we set up for a perspective view
	glMatrixMode(GL_PROJECTION);        // Select the Projection Matrix
	glLoadIdentity();                   // Reset(init) the Projection Matrix

	gluPerspective(45.0f, w/h, 0.1f, 100.0f);       
	glMatrixMode(GL_MODELVIEW);         // Select the modelview matrix
	glLoadIdentity();                   // Reset (init) the modelview matrix
	
	gluLookAt(	0.0f,					// GLdouble eyex
				0.0f,					// GLdouble eyey
				30.0f,					// GLdouble eyez
				0.0f,					// GLdouble centerx
				0.0f,					// GLdouble centery
				0.0f,					// GLdouble centerz
				0.0f,					// GLdouble upx
				1.0f,					// GLdouble upy
				0.0f);					// GLdouble upz

	
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glShadeModel(GL_SMOOTH);             // Enable smooth shading
	glClearDepth(1.0f);                  // Clear depth buffer
	glEnable(GL_DEPTH_TEST);             // Enables depth test
	glDepthFunc(GL_LEQUAL);              // Type of depth test to perform


	// Use static variables for our xspeed and yspeed to try and avoid 
	// haveing globals
	// xspeed and yspeed are rotation values in radians - as we can drag the mouse
	// fast and it will keep track of a veleocty speed in variables g_fVelY and
	// g_fVelX, which can be used to keep rotating the shape...so you can drag
	// and rotate the shape quick and let go and it will keep rotating.
	static float xspeed = 0.0f;			 
	xspeed += g_fVelX;

	static float yspeed = 0.0f;
	yspeed += g_fVelY;

	// Rotation variables
	float spinx = g_fSpinY - yspeed;
	float spiny = g_fSpinX - xspeed;
	float spinz = 0;

	// OpenGL Translation and Roation API's
	glTranslated(0, 0, -g_fZoom);
	glRotatef(spinx, 1.0f, 0.0f, 0.0f);
	glRotatef(spiny, 0.0f, 1.0f, 0.0f);
	glRotatef(spinz, 0.0f, 0.0f, 1.0f);

}// End UpdateCameara(..)

/***************************************************************************/
/*                                                                         */
/*  DrawCube()                                                             */
/*                                                                         */
/*  Nice to be able to see where our camera is, or if we want to draw      */
/*  a 3d position in space...so I added this little debug function, which  */
/*  we can call with a position, and it simply draw a cube at the position */
/*  we pass in.                                                            */
/*                                                                         */
/***************************************************************************/

void DrawCube(VECTOR3& pos)
{
	const float s = 0.1f;
	glTranslatef(pos.x, pos.y, pos.z);	// Translate Into The Screen 7.0 Units
	glBegin(GL_QUADS);					// Draw The Cube Using quads
		glColor3f(0.0f,1.0f,0.0f);	// Color Blue
		glVertex3f( 1.0f*s, 1.0f*s,-1.0f*s);	// Top Right Of The Quad (Top)
		glVertex3f(-1.0f*s, 1.0f*s,-1.0f*s);	// Top Left Of The Quad (Top)
		glVertex3f(-1.0f*s, 1.0f*s, 1.0f*s);	// Bottom Left Of The Quad (Top)
		glVertex3f( 1.0f*s, 1.0f*s, 1.0f*s);	// Bottom Right Of The Quad (Top)
		glColor3f(1.0f,0.5f,0.0f);	// Color Orange
		glVertex3f( 1.0f*s,-1.0f*s, 1.0f*s);	// Top Right Of The Quad (Bottom)
		glVertex3f(-1.0f*s,-1.0f*s, 1.0f*s);	// Top Left Of The Quad (Bottom)
		glVertex3f(-1.0f*s,-1.0f*s,-1.0f*s);	// Bottom Left Of The Quad (Bottom)
		glVertex3f( 1.0f*s,-1.0f*s,-1.0f*s);	// Bottom Right Of The Quad (Bottom)
		glColor3f(1.0f,0.0f,0.0f);	// Color Red	
		glVertex3f( 1.0f*s, 1.0f*s, 1.0f*s);	// Top Right Of The Quad (Front)
		glVertex3f(-1.0f*s, 1.0f*s, 1.0f*s);	// Top Left Of The Quad (Front)
		glVertex3f(-1.0f*s,-1.0f*s, 1.0f*s);	// Bottom Left Of The Quad (Front)
		glVertex3f( 1.0f*s,-1.0f*s, 1.0f*s);	// Bottom Right Of The Quad (Front)
		glColor3f(1.0f,1.0f,0.0f);	// Color Yellow
		glVertex3f( 1.0f*s,-1.0f*s,-1.0f*s);	// Top Right Of The Quad (Back)
		glVertex3f(-1.0f*s,-1.0f*s,-1.0f*s);	// Top Left Of The Quad (Back)
		glVertex3f(-1.0f*s, 1.0f*s,-1.0f*s);	// Bottom Left Of The Quad (Back)
		glVertex3f( 1.0f*s, 1.0f*s,-1.0f*s);	// Bottom Right Of The Quad (Back)
		glColor3f(0.0f,0.0f,1.0f);	// Color Blue
		glVertex3f(-1.0f*s, 1.0f*s, 1.0f*s);	// Top Right Of The Quad (Left)
		glVertex3f(-1.0f*s, 1.0f*s,-1.0f*s);	// Top Left Of The Quad (Left)
		glVertex3f(-1.0f*s,-1.0f*s,-1.0f*s);	// Bottom Left Of The Quad (Left)
		glVertex3f(-1.0f*s,-1.0f*s, 1.0f*s);	// Bottom Right Of The Quad (Left)
		glColor3f(1.0f,0.0f,1.0f);	// Color Violet
		glVertex3f( 1.0f*s, 1.0f*s,-1.0f*s);	// Top Right Of The Quad (Right)
		glVertex3f( 1.0f*s, 1.0f*s, 1.0f*s);	// Top Left Of The Quad (Right)
		glVertex3f( 1.0f*s,-1.0f*s, 1.0f*s);	// Bottom Left Of The Quad (Right)
		glVertex3f( 1.0f*s,-1.0f*s,-1.0f*s);	// Bottom Right Of The Quad (Right)
	glEnd();							// End Drawing The Cube
}// End DrawCube(..)

float g_minCamDist = 1.2f;


/***************************************************************************/
/*                                                                         */
/*  GetOrbitPosA()                                                         */
/*                                                                         */
/*  Basic boring orbit
/***************************************************************************/
VECTOR3 GetOrbitPosA(float val, bool adjustSpeed = false)
{
	float x0 = sinf( val );
	float z0 = cosf( val );

	// Minimum orbit distance
	float len = g_minCamDist;
	
	// Note, len is the distance from the planet, and the 1.5 is so the orbit
	// is skewed, so its not circular! ...a egg like orbit is cooler I think.

	x0 = x0 * len;
	z0 = z0 * len * 2.5f;

	VECTOR3 pos (x0, 0, z0);

	return pos;
}// End GetOrbitPosA(..)

/***************************************************************************/
/*                                                                         */
/*  GetOrbitPosB()                                                         */
/*                                                                         */
/***************************************************************************/
// Info:
// adjustSpeed variable is so we can account for the same speed for the 
// different orbits, as by default, each complete orbit takes 2PI, no matter 
// how complex,so by adding in the adjustSpeed, we can gurantee that each 
// orbit val will give us more or less the same different in change along 
// the orbit path.
//
/***************************************************************************/

VECTOR3 GetOrbitPosB(float val, bool adjustSpeed = false)
{
	if (adjustSpeed)
	{
		val /= 3.0f;
	}

	float x0 = sinf( val * 3 );
	float z0 = cosf( val );
	float y0 = cosf( val * 3 );

	// Minimum orbit distance
	float len = g_minCamDist;
	
	// Note, len is the distance from the planet, and the 1.5 is so the orbit
	// is skewed, so its not circular! ...a egg like orbit is cooler I think.

	x0 = x0 * len;
	z0 = z0 * len * 2.5f;
	y0 = y0 * len;

	VECTOR3 pos (x0, y0, z0);

	return pos;
}// End GetOrbitPosB(..)

/***************************************************************************/
/*                                                                         */
/*  GetOrbitPosC()                                                         */
/*                                                                         */
/***************************************************************************/
VECTOR3 GetOrbitPosC(float val, bool adjustSpeed = false)
{
	if (adjustSpeed)
	{
		val /= 4.0f;
	}

	float x0 = sinf( val * 4 );
	float z0 = cosf( val * 4 );
	float y0 = cosf( val * 5 );

	// Minimum orbit distance
	float len = g_minCamDist;
	
	// Note, len is the distance from the planet, and the 1.5 is so the orbit
	// is skewed, so its not circular! ...a egg like orbit is cooler I think.

	x0 = x0 * len;
	z0 = z0 * len * 2.5f;
	y0 = y0 * len;

	VECTOR3 pos (x0, y0, z0);

	return pos;
}// End GetOrbitPosC(..)

/***************************************************************************/
/*                                                                         */
/*  GetOrbitPos()                                                          */
/*                                                                         */
/***************************************************************************/
VECTOR3 GetOrbitPos(float val, bool adjustSpeed = false)  // val -> 0 to 2PI
{
	VECTOR3 ret = VECTOR3(0,0,0);

	switch (g_OrbitSelected)
	{
		case ORBITA:
		{
			return GetOrbitPosA(val, adjustSpeed);
		}
		break;

		case ORBITB:
		{
			return GetOrbitPosB(val, adjustSpeed);
		}
		break;

		case ORBITC:
		{
			return GetOrbitPosC(val, adjustSpeed);
		}
		break;

		default:
		{
			// Error unknown orbit!
		}
	}

	return VECTOR3(0,0,0);
}// End GetOrbitPos(..)

/***************************************************************************/
/*                                                                         */
/*  DrawOrbit()                                                            */
/*                                                                         */
/*  Its so much cooler if you can see the orbit that your camera is going  */
/*  to follow...so when where watching it from a 3rd person...we draw a    */
/*  line for the cameras path...the path is nothing more than a a sin/cos  */
/*  with we itterate across 2PI around the x and z axis.                   */
/*                                                                         */
/***************************************************************************/
void DrawOrbit()
{
	int steps = 50;
	float inc = (float)g_TWOPI / steps;

	for (int i=0; i<steps; i++)
	{
		VECTOR3 p0 = GetOrbitPos( i*inc );
		VECTOR3 p1 = GetOrbitPos( (i+1)*inc );

		glBegin(GL_LINES);
		glVertex3f(p0.x, p0.y, p0.z);
		glVertex3f(p1.x, p1.y, p1.z);
		glEnd();
	}

	// Lets draw the current position on the path that the camera is on
	if (!g_bOrbitCam)
	{
		DrawCube(g_OribitPos);
	}

}// End DrawOrbit(..)


/***************************************************************************/
/*                                                                         */
/*  UpdateOrbitPosition()                                                  */ 
/*                                                                         */
/*  I think the function name speaks for itself!  We simply call this      */
/*  this function from the main loop to update the position of the camera  */
/*  as it orbits the planet...it follows the same path that we create in   */
/*  DrawOrbit(..) above.                                                   */
/*                                                                         */
/***************************************************************************/
void UpdateOrbitPosition()
{
	// Need to add some timing information, as not everyones pc runs at the
	// same speed!
	static float oldTime = (float)GetTickCount();
	float timeNow = (float)GetTickCount() + 1.0f;

	float delay = (timeNow - oldTime);
	oldTime = timeNow;

	delay /= 1000.0f;

	static float val = 0.0f;
	val += g_OrbitSpeed * delay;

	g_OribitPos		= GetOrbitPos(val, true);
	g_OribitPosNext = GetOrbitPos(val + 0.1f, true);

}// End UpdateOrbitPosition(..)


/***************************************************************************/
/*                                                                         */
/* UpdateOrbitCam()                                                        */
/*                                                                         */
/***************************************************************************/
void UpdateOrbitCam()
{
	RECT rect;
	::GetClientRect(g_hwnd, &rect);

	int w = rect.right - rect.left;
	int h = rect.bottom - rect.top;

	// If you get 0/0 it means the window is minimized, jsut default to some
	// value so opengl doesn't crash!
	if (w==0 || h==0)
	{
		w=640;
		h=480;
	}

	// Current viewport:x, y, width, height
	glViewport(0, 0, w, h);    
 
	// Next we set up for a perspective view
	glMatrixMode(GL_PROJECTION);	         // Select the Projection Matrix
	glLoadIdentity();           	         // Reset(init) the Projection Matrix

	gluPerspective(45.0f, w/h, 0.1f, 1000.0f);       
	glMatrixMode(GL_MODELVIEW);          	// Select the modelview matrix
	glLoadIdentity();                    	// Reset (init) the modelview matrix
	

	// Need to add a bit of extra code, so we are looking ahead, over the
	// earths surface when we get to close to the planet...as if where
	// running along hte earths surface.
	VECTOR3 target = VECTOR3(0,0,0);
	float distCentrePlanet = g_OribitPos.length();

	// Starts the transistion effect for the camere when its within 2.0f of the
	// earths surface, remember the sphere has a radius of 1
	if ( distCentrePlanet < 2.0f )
	{
		// Blend camera look at, so its smoother...more work, but looks nicer

		float lerp = 1.0f;
		if (distCentrePlanet>g_minCamDist)
		{
			lerp = (2.0f - distCentrePlanet) / (2.0f - g_minCamDist);
		}

		VECTOR3 surfTarget = g_OribitPosNext - g_OribitPos;
		surfTarget.normalize();

		target = target + lerp*( surfTarget - target );
	}

	gluLookAt(	g_OribitPos.x,				// GLdouble eyex
				g_OribitPos.y,				// GLdouble eyey
				g_OribitPos.z,				// GLdouble eyez
				target.x,					// GLdouble centerx
				target.y,					// GLdouble centery
				target.z,					// GLdouble centerz
				0.0f,						// GLdouble upx
				1.0f,						// GLdouble upy
				0.0f);						// GLdouble upz

	
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glShadeModel(GL_SMOOTH);	   			// Enable smooth shading
	glClearDepth(1.0f);     	   			// Clear depth buffer
	glEnable(GL_DEPTH_TEST);	   			// Enables depth test
	glDepthFunc(GL_LEQUAL); 	   			// Type of depth test to perform



	// OpenGL Translation and Roation API's
	glTranslated(0, 0, 0);
	glRotatef(0, 1.0f, 0.0f, 0.0f);
	glRotatef(0, 0.0f, 1.0f, 0.0f);
	glRotatef(0, 0.0f, 0.0f, 1.0f);

}// End UpdateOrbitCam(..)


/***************************************************************************/
/*                                                                         */
/* render()                                                                */
/* Our main render loop, which gets called over and over agian to do our   */
/* drawing...clears the screen, draws the data, then presents it..         */
/*                                                                         */
/***************************************************************************/
void render()
{
	glLoadIdentity();
	glScalef(1.0f, 1.0f, 1.0f);				
	glTranslatef(0.0f,0.0f,0.0f);
	glColor3f (1.0f, 1.0f, 1.0f);

	if (g_bOrbitCam)
	{
		UpdateOrbitCam();
	}
	else
	{
		UpdateCamera();										// Update view/world mat
	}

	KeyBoard();												// Get keyboard input

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);					// Set background to black
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear screen

	DrawTexSphere();										// Create & Render Shape

	DrawOrbit();

	UpdateOrbitPosition();

	// Display information to the user - show our user the eqation which build
	// they are running, and possibly any keyboard information
	static char buf[300];
	sprintf(buf, "Earth Demo (v01)");
	static float xx = 100;
	static float yy = 100;
	DebugPrint(buf, xx, yy);

}//End render()
